@@ -1,5 +1,5 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-//     Copyright (C) 2012-2018 Reza Mohammadi                                                      |
+//     Copyright (C) 2012 - 2019  Reza Mohammadi                                                   |
 //                                                                                                 |
 //     This file is part of BDgraph package.                                                       |
 //                                                                                                 |
@@ -14,118 +14,48 @@
    
 extern "C" {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-// Calculating mean for copula function
+// Computing mean for copula function
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 void get_mean( double Z[], double K[], double *mu_ij, double *sigma, int *i, int *j, int *n, int *p )
 {
     int k, dim = *p, number = *n, row = *i, col = *j, jxp = col * dim;
     double mu = 0.0;
     
-    for( k = 0; k < col; k++ ) 
-        mu += Z[ k * number + row ] * K[ jxp + k ];
-    
-    for( k = col + 1; k < dim; k++ ) 
-        mu += Z[ k * number + row ] * K[ jxp + k ];
+    for( k = 0;       k < col; k++ ) mu += Z[ k * number + row ] * K[ jxp + k ];
+    for( k = col + 1; k < dim; k++ ) mu += Z[ k * number + row ] * K[ jxp + k ];
     
     *mu_ij = - mu * *sigma;
 }
-
+    
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-// Calculating bounds for copula function
+// Computing bounds for copula function
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 void get_bounds( double Z[], int R[], double *lb, double *ub, int *i, int *j, int *n )
 {
-	int kj, ij, row = *i, col = *j;
-	double low_b = -1e308, upper_b = +1e308;
-
-	for( int k = 0, number = *n; k < number; k++ )
-	{
-		kj = col * number + k;
-		ij = col * number + row;
-		
-		// if( R[k, j] < R[i, j] ) lb = max( Z[ k, j], lb )
-		// if( R[k, j] > R[i, j] ) ub = min( Z[ k, j], ub )										
-		if( R[ kj ] < R[ ij ] ) 
-			low_b = max( Z[ kj ], low_b );	
-		else if( R[ kj ] > R[ ij ] ) 
-			upper_b = min( Z[ kj ], upper_b );
-	}
-
-	*lb = low_b;
-	*ub = upper_b;	
+    int kj, col = *j, number = *n, ij = col * number + *i;
+    double low_b = -1e308, upper_b = +1e308;
+    
+    for( int k = 0; k < number; k++ )
+    {
+        kj = col * number + k;
+        
+        if( R[ kj ] < R[ ij ] ) 
+            low_b = max( Z[ kj ], low_b );	
+        else if( R[ kj ] > R[ ij ] ) 
+            upper_b = min( Z[ kj ], upper_b );
+    }
+    
+    *lb = low_b;
+    *ub = upper_b;	
 }
- 
+    
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-// copula part
+// copula for BDMCMC sampling algorithm
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 void copula( double Z[], double K[], int R[], int not_continuous[], int *n, int *p )
-{
-    int number = *n, dim = *p, dimp1 = dim + 1, i, j, jxn;
-    double sigma, sd_j, mu_ij, lb, ub, runif_value, pnorm_lb, pnorm_ub;
-    
-    //GetRNGstate();
-    for( j = 0; j < dim; j++ )
-    {
-        if( not_continuous[ j ] )
-        {
-            jxn = j * number;
-            
-            sigma = 1.0 / K[ j * dimp1 ]; // 1.0 / K[ j * dim + j ];
-            sd_j  = sqrt( sigma );
-            
-            for( i = 0; i < number; i++ )
-            {   
-                get_mean( Z, K, &mu_ij, &sigma, &i, &j, &number, &dim );
-                
-                get_bounds( Z, R, &lb, &ub, &i, &j, &number );
-                
-                pnorm_lb     = Rf_pnorm5( lb, mu_ij, sd_j, TRUE, FALSE );
-                pnorm_ub     = Rf_pnorm5( ub, mu_ij, sd_j, TRUE, FALSE );
-                //runif_value = runif( pnorm_lb, pnorm_ub );
-                runif_value  = pnorm_lb + unif_rand() * ( pnorm_ub - pnorm_lb );
-                Z[ jxn + i ] = Rf_qnorm5( runif_value, mu_ij, sd_j, TRUE, FALSE );
-            }
-        }
-    }
-    //PutRNGstate();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-// Calculating bounds for copula function with missing data 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-void get_bounds_NA( double Z[], int R[], double *lb, double *ub, int *i, int *j, int *n )
-{
-	int kj, ij, row = *i, col = *j;
-	double low_b = -1e308, upper_b = +1e308;
-
-	for( int k = 0, number = *n; k < number; k++ )
-	{
-		kj = col * number + k;
-		ij = col * number + row;
-		
-		if( R[kj] != 0 )
-		{
-			// if( R[k, j] < R[i, j] ) lb = max( Z[ k, j], lb )
-			// if( R[k, j] > R[i, j] ) ub = min( Z[ k, j], ub )										
-			if( R[ kj ] < R[ ij ] ) 
-				low_b = max( Z[ kj ], low_b );	
-			else if( R[ kj ] > R[ ij ] ) 
-				upper_b = min( Z[ kj ], upper_b );
-		}
-	}
-	
-	*lb = low_b;
-	*ub = upper_b;		
-}
- 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-// copula part for missing data
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
-void copula_NA( double Z[], double K[], int R[], int not_continuous[], int *n, int *p )
 {
     int number = *n, dim = *p, nxp = number * dim, dimp1 = dim + 1;
- 
-    //GetRNGstate();
+    
     #pragma omp parallel
     {	
         double sigma, sd_j, mu_ij, lb, ub, runif_value, pnorm_lb, pnorm_ub;
@@ -137,13 +67,13 @@ void copula_NA( double Z[], double K[], int R[], int not_continuous[], int *n, i
             j = counter / number;
             i = counter % number;
             
-            sigma = 1.0 / K[ j * dimp1 ]; // 1.0 / K[ j * dim + j ];
-            sd_j  = sqrt( sigma );
-            
-            get_mean( Z, K, &mu_ij, &sigma, &i, &j, &number, &dim );
-            
-            if( R[ counter ] != 0 )
+            if( not_continuous[ j ] )
             {
+                sigma = 1.0 / K[ j * dimp1 ]; // 1.0 / K[ j * dim + j ];
+                sd_j  = sqrt( sigma );
+                
+                get_mean( Z, K, &mu_ij, &sigma, &i, &j, &number, &dim );
+                
                 get_bounds_NA( Z, R, &lb, &ub, &i, &j, &number );
                 
                 pnorm_lb     = Rf_pnorm5( lb, mu_ij, sd_j, TRUE, FALSE );
@@ -151,12 +81,75 @@ void copula_NA( double Z[], double K[], int R[], int not_continuous[], int *n, i
                 //runif_value = runif( pnorm_lb, pnorm_ub );
                 runif_value  = pnorm_lb + unif_rand() * ( pnorm_ub - pnorm_lb );
                 Z[ counter ] = Rf_qnorm5( runif_value, mu_ij, sd_j, TRUE, FALSE );
-            } 
-            else 
-                Z[ counter ] = mu_ij + norm_rand() * sd_j;  // Z[counter] = rnorm( mu_ij, sd_j );
+            }
+        }
+    }
+}
+    
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
+// Computing bounds for copula function for data with missing values 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
+void get_bounds_NA( double Z[], int R[], double *lb, double *ub, int *i, int *j, int *n )
+{
+    int kj, number = *n, col = *j, ij = col * number + *i;
+    double low_b = -1e308, upper_b = +1e308;
+    
+    for( int k = 0; k < number; k++ )
+    {
+        kj = col * number + k;
+       
+        if( R[ kj ] != 0 )
+        {
+            if( R[ kj ] < R[ ij ] ) 
+                low_b = max( Z[ kj ], low_b );	
+            else if( R[ kj ] > R[ ij ] ) 
+                upper_b = min( Z[ kj ], upper_b );
+        }
+    }
+    
+    *lb = low_b;
+    *ub = upper_b;		
+}
+    
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
+// copula for data with missing values 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
+void copula_NA( double Z[], double K[], int R[], int not_continuous[], int *n, int *p )
+{
+    int number = *n, dim = *p, nxp = number * dim, dimp1 = dim + 1;
+    
+    #pragma omp parallel
+    {	
+        double sigma, sd_j, mu_ij, lb, ub, runif_value, pnorm_lb, pnorm_ub;
+        int i, j;
+        
+        #pragma omp for
+        for( int counter = 0; counter < nxp; counter++ )
+        {   
+            j = counter / number;
+            i = counter % number;
+            
+            if( not_continuous[ j ] )
+            {
+                sigma = 1.0 / K[ j * dimp1 ]; // 1.0 / K[ j * dim + j ];
+                sd_j  = sqrt( sigma );
+                
+                get_mean( Z, K, &mu_ij, &sigma, &i, &j, &number, &dim );
+                
+                if( R[ counter ] != 0 )
+                {
+                    get_bounds_NA( Z, R, &lb, &ub, &i, &j, &number );
+                    
+                    pnorm_lb     = Rf_pnorm5( lb, mu_ij, sd_j, TRUE, FALSE );
+                    pnorm_ub     = Rf_pnorm5( ub, mu_ij, sd_j, TRUE, FALSE );
+                    //runif_value = runif( pnorm_lb, pnorm_ub );
+                    runif_value  = pnorm_lb + unif_rand() * ( pnorm_ub - pnorm_lb );
+                    Z[ counter ] = Rf_qnorm5( runif_value, mu_ij, sd_j, TRUE, FALSE );
+                }else
+                    Z[ counter ] = mu_ij + norm_rand() * sd_j;  // rnorm( mu_ij, sd_j );
+            }
         }
     }
-    //PutRNGstate();
 }
     
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
@@ -164,30 +157,28 @@ void copula_NA( double Z[], double K[], int R[], int not_continuous[], int *n, i
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 void get_Ds( double K[], double Z[], int R[], int not_continuous[], double D[], double Ds[], double S[], int *gcgm, int *n, int *p )
 {
-	int gcgm_check = *gcgm, dim = *p, pxp = dim * dim;
+	int dim = *p;
 
-	// if( gcgm_check == 0 ) copula( Z, K, R, n, &dim ); else	copula_NA( Z, K, R, n, &dim );
-	( gcgm_check == 0 ) ? copula( Z, K, R, not_continuous, n, &dim ) : copula_NA( Z, K, R, not_continuous, n, &dim );
+	( *gcgm == 0 ) ? copula( Z, K, R, not_continuous, n, &dim ) : copula_NA( Z, K, R, not_continuous, n, &dim );
 	
-	// S <- t(Z) %*% Z
-	// Here, I'm using Ds instead of S, for saving memory
+	// S <- t(Z) %*% Z; NOTE, I'm using Ds instead of S, for saving memory
 	double alpha = 1.0, beta  = 0.0;
 	char transA = 'T', transB = 'N';
-	// LAPACK function to compute  C := alpha * A * B + beta * C
-	//        DGEMM ( TRANSA,  TRANSB, M, N, K,  ALPHA, A,LDA,B, LDB,BETA, C, LDC )																				
 	F77_NAME(dgemm)( &transA, &transB, &dim, &dim, n, &alpha, Z, n, Z, n, &beta, &S[0], &dim );		
-	// Ds = D + S
-	for( int i = 0; i < pxp ; i++ ) Ds[ i ] = D[ i ] + S[ i ];		
+	
+    #pragma omp parallel for
+	for( int i = 0; i < dim * dim; i++ ) 
+	    Ds[ i ] = D[ i ] + S[ i ];		
 }
-
+    
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 // Calculating Ts = chol( solve( Ds ) ) for the BDMCMC sampling algorithm
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |
 void get_Ts( double Ds[], double Ts[], double inv_Ds[], double copy_Ds[], int *p )
 {
-	int dim = *p, pxp = dim * dim;
+	int dim = *p; 
 
-	memcpy( &copy_Ds[0], Ds, sizeof( double ) * pxp );
+	memcpy( &copy_Ds[0], Ds, sizeof( double ) * dim * dim );
 	
 	inverse( &copy_Ds[0], &inv_Ds[0], &dim );	
 
